@@ -1,8 +1,7 @@
-use std::mem::take;
+use std::{mem::take};
 
 use proc_macro::{Delimiter, Spacing, TokenTree};
 
-#[proc_macro]
 /// Duplicates the given code and substitutes specific identifiers for different code snippets in each duplicate.
 ///
 /// ## Usage
@@ -223,35 +222,29 @@ use proc_macro::{Delimiter, Spacing, TokenTree};
 ///     }
 /// }
 /// ```
+#[proc_macro]
 pub fn akin(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
-    let mut vars: Vec<(String, Vec<String>)> = Vec::new();
+    let mut vars: Vec<(String, Vec<String>)> = Vec::with_capacity(2);
     //panic!("Tokens: {input:#?}");
     let mut tokens = input.into_iter();
-
+    
     let mut first = tokens.next().expect("akin: expected code to duplicate");
-    let mut second = tokens
-        .next()
-        .unwrap_or_else(|| panic!("akin: expected code to duplicate"));
-    while matches!(&first, TokenTree::Ident(ident) if ident.to_string() == "let")
-        && matches!(&second, TokenTree::Punct(punct) if punct.to_string() == "&")
+    let mut second = tokens.next().expect("akin: expected code to duplicate");
+    while matches!(&first, TokenTree::Ident(id) if id.to_string() == "let")
+        && matches!(&second, TokenTree::Punct(p) if p.as_char() == '&')
     {
         vars.push(parse_var(&mut tokens, &vars));
-        first = tokens
-            .next()
-            .unwrap_or_else(|| panic!("akin: expected code to duplicate"));
-        second = tokens
-            .next()
-            .unwrap_or_else(|| panic!("akin: expected code to duplicate"));
+        first = tokens.next().expect("akin: expected code to duplicate");
+        second = tokens.next().expect("akin: expected code to duplicate");
     }
 
-    let mut previous = second.clone();
-
-    let init = fold(
-        fold(String::new(), first, &mut previous),
+    let mut previous = first.clone();
+    let init = fold_tt(
+        fold_tt(String::new(), first, &mut previous),
         second,
         &mut previous,
     );
-    let out_raw = tokens.fold(init, |acc, tt| fold(acc, tt, &mut previous));
+    let out_raw = tokens.fold(init, |acc, tt| fold_tt(acc, tt, &mut previous));
     let out = duplicate(&out_raw, &vars);
 
     //let tokens = format!("proc_macro: {:#?}", input.into_iter().collect::<Vec<_>>());
@@ -269,44 +262,82 @@ fn parse_var(
         "*{}",
         tokens.next().expect("akin: expected code to duplicate")
     );
+
     let mut prev = tokens.next().expect("akin: expected code to duplicate"); // skip '='
     let mut values: Vec<String> = Vec::new();
-    let group = tokens.next().expect("akin: expected code to duplicate");
-    if let TokenTree::Group(group) = &group {
-        if group.delimiter() == Delimiter::Bracket {
-            let mut add = String::new();
-            for var in group.stream() {
-                let txt = take(&mut add);
-                let new = match &var {
-                    TokenTree::Group(g) if g.delimiter() == Delimiter::Brace => 
-                        g
-                        .stream()
-                        .into_iter()
-                        .fold(String::new(), |acc, tt| fold(acc, tt, &mut prev)),
-                    TokenTree::Punct(p) if p.as_char() != ',' => {
-                        add = var.to_string();
-                        continue;
-                    }
+    let group =
+        if let TokenTree::Group(g) = tokens.next().expect("akin: expected code to duplicate") {
+            g
+        } else {
+            return (name, values);
+        };
+
+    if group.delimiter() == Delimiter::Bracket {
+        let mut add = String::new();
+
+        let mut stream = group.stream().into_iter();
+        
+        while let Some(var) = stream.next() {
+            let mut var = var;
+            let mut new = String::new();
+            while !matches!(&var, TokenTree::Punct(p) if p.as_char() == ',') {
+                new += &match &var {
+                    TokenTree::Group(g) if g.delimiter() == Delimiter::Brace => g
+                    .stream()
+                    .into_iter()
+                    .fold(String::new(), |acc, tt| fold_tt(acc, tt, &mut prev)),
+
                     _ => var.to_string(),
                 };
                 
-                if new == "NONE" {
-                    values.push(String::new())
-                } else if new != "," {
-                    values.push(duplicate(&(txt + &new), vars));
+                if let Some(v) = stream.next() {
+                    var = v;
+                } else {
+                    break;
                 }
             }
-        } else {
-            let fold = group
-                .stream()
-                .into_iter()
-                .fold(String::new(), |acc, tt| fold(acc, tt, &mut prev));
-            values.push(duplicate(&fold, vars));
-        }
 
-        if !matches!(tokens.next(), Some(TokenTree::Punct(p)) if p.as_char() == ';') {
-            panic!("akin: expected ';' on {}'s declaration end", name.replacen('*', "&", 1));
+            if new == "NONE" {
+                values.push(String::new())
+            } else {
+                values.push(duplicate(&(take(&mut add) + &new), vars));
+            }
         }
+        
+        /*
+        for var in group.stream() {
+            let new = match &var {
+                // Case {group} => Variable is code, braces need to be skipped
+                // Case punct   => Variable is punctuation, it will be grouped with the next variable. Workaround for characters like '-' or '!'
+                // Case _       => Variable is a value, it is printed normally.
+                TokenTree::Group(g) if g.delimiter() == Delimiter::Brace => g
+                    .stream()
+                    .into_iter()
+                    .fold(String::new(), |acc, tt| fold_tt(acc, tt, &mut prev)),
+                _ => var.to_string(),
+            };
+
+            if new == "NONE" {
+                values.push(String::new())
+            } else if new != "," {
+                values.push(duplicate(&(take(&mut add) + &new), vars));
+            }
+        }
+        */
+    } else {
+        let fold = group
+            .stream()
+            .into_iter()
+            .fold(String::new(), |acc, tt| fold_tt(acc, tt, &mut prev));
+        values.push(duplicate(&fold, vars));
+    }
+
+    if !matches!(tokens.next(), Some(TokenTree::Punct(p)) if p.as_char() == ';') {
+        panic!(
+            "akin: expected ';' on end of '&{}' declaration",
+            // VALIDITY: I'm only removing the first character, which is always '*', so it will never fail
+            unsafe { std::str::from_utf8_unchecked(&name.as_bytes()[1..]) }
+        );
     }
 
     (name, values)
@@ -314,18 +345,17 @@ fn parse_var(
 
 fn duplicate(stream: &str, vars: &[(String, Vec<String>)]) -> String {
     let (vars, times) = get_used_vars(stream, vars);
-    let mut out = String::new();
+    let mut out = String::with_capacity(stream.len() * times);
     for i in 0..times {
-        let mut temp = stream.to_owned();
+        out += stream;
         for var in &vars {
-            temp = temp.replace(
+            out = out.replace(
                 &var.0,
                 var.1.get(i).unwrap_or_else(|| var.1.last().unwrap()),
             )
         }
-        out += &temp;
     }
-    
+
     if out.is_empty() {
         stream.into()
     } else {
@@ -333,16 +363,16 @@ fn duplicate(stream: &str, vars: &[(String, Vec<String>)]) -> String {
     }
 }
 
-fn get_used_vars(
+fn get_used_vars<'a>(
     stream: &str,
-    vars: &[(String, Vec<String>)],
-) -> (Vec<(String, Vec<String>)>, usize) {
-    let mut used = Vec::new();
+    vars: &'a [(String, Vec<String>)],
+) -> (Vec<&'a (String, Vec<String>)>, usize) {
+    let mut used = Vec::with_capacity(vars.len());
     let mut times = 0;
 
     for var in vars {
         if stream.contains(&var.0) {
-            used.push(var.clone());
+            used.push(var);
             times = times.max(var.1.len());
         }
     }
@@ -359,28 +389,24 @@ fn get_delimiters(delimiter: Delimiter) -> (char, char) {
     }
 }
 
-fn fold(a: String, tt: TokenTree, prev: &mut TokenTree) -> String {
-    let ret = if let TokenTree::Group(group) = &tt {
-        let (start, end) = get_delimiters(group.delimiter());
-        let group = group
+fn fold_tt(a: String, tt: TokenTree, prev: &mut TokenTree) -> String {
+    let ret = if let TokenTree::Group(g) = &tt {
+        let (start, end) = get_delimiters(g.delimiter());
+        let group = g
             .stream()
             .into_iter()
-            .fold(String::new(), |acc, tt| fold(acc, tt, prev));
+            .fold(String::new(), |acc, tt| fold_tt(acc, tt, prev));
         format!("{a}{start}{group}{end}")
     } else if matches!(&tt, TokenTree::Punct(p) if p.as_char() == '~') {
         a // skip character
-    } else if let TokenTree::Punct(p) = prev {
-        // Case '*' => To make variable formatting easier ('*var' instead of '* var')
+    } else if matches!(&prev, TokenTree::Punct(p) if p.spacing() == Spacing::Joint || matches!(p.as_char(), '*' | '~')) {
+        // Case '*' => To make variable formatting simpler ('*var' instead of '* var')
         // Case '~' => Behaviour of the '~' modifier
-        if p.spacing() == Spacing::Joint || matches!(p.as_char(), '*' | '~') {
-            format!("{a}{tt}")
-        } else {
-            format!("{a} {tt}")
-        }
+        format!("{a}{tt}")
     } else {
         format!("{a} {tt}")
     };
-
+    
     *prev = tt;
     ret
 }
